package kriptografija;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.Random;
import java.lang.Math;
import java.math.BigDecimal;
import java.math.RoundingMode;

import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

public class Dokument {
	
	public static final int IK = 3;
	public static final int P = 4;
	
	private final String MSG_NEPOSTOJECI_FAJL = "nepostojeci fajl";
	private final String MSG_IO_EXCEPTION_READ = "greska pri citanju";	
	private final String MSG_IO_EXCEPTION_WRITE = "greska pri upisu";	
	
	private String naziv_jezika;
	private String definicija_pisma;
	private int duzina_pisma;
	private char mala_slova[];
	private char velika_slova[];
	private int broj_pojavljivanja[];
	private double interval_poverenja_donja_granica;
	private double interval_poverenja_gornja_granica;	
	
	private StringBuffer tekst;	

	public Dokument(String naziv_jezika, String definicija_pisma, String interval_poverenja_donja_granica, String interval_poverenja_gornja_granica) {
		ucitajJezik(naziv_jezika, definicija_pisma, interval_poverenja_donja_granica, interval_poverenja_gornja_granica);
	}	
	
	public Dokument(File jezik, String charsetName) {
		if ((jezik.exists()) && (jezik.isFile())) {
			try {
				FileInputStream fis = new FileInputStream(jezik);
				InputStreamReader isr = new InputStreamReader(fis, charsetName);		
				BufferedReader br = new BufferedReader(isr);
				
				ucitajJezik(br.readLine(), br.readLine(), br.readLine(), br.readLine());
				
				br.close();
				isr.close();
				fis.close();
			} catch (IOException e) {
				System.out.println(MSG_IO_EXCEPTION_READ);				
			}
		} else {
			System.out.println(MSG_NEPOSTOJECI_FAJL);
		}
	}
	
	public void dodajTekst(File fajl) {
		if ((fajl.exists()) && (fajl.isFile())) {
			try {
				FileInputStream fis = new FileInputStream(fajl);
				InputStreamReader isr = new InputStreamReader(fis, "UTF-16");
				int i;
				while ((i = isr.read()) != -1) {
					tekst.append(obradiKarakter((char)i));
				}
				isr.close();
				fis.close();
			} catch (IOException e) {
				System.out.println(MSG_IO_EXCEPTION_READ);				
			}
		} else {
			System.out.println(MSG_NEPOSTOJECI_FAJL);
		}
		popuniPojavljivanja();
	}
	public void dodajTekst(StringBuffer sb) {
		for (int i = 0; i < sb.length(); i++) {
			tekst.append(obradiKarakter(sb.charAt(i)));
		}
		popuniPojavljivanja();		
	}
	public void dodajTekst(String string) {
		for (int i = 0; i < string.length(); i++) {
			tekst.append(obradiKarakter(string.charAt(i)));
		}
		popuniPojavljivanja();		
	}
	public void dodajTekst(char karakter) {
		tekst.append(obradiKarakter(karakter));
		popuniPojavljivanja();		
	}	
	
	public String nazivJezika() {
		return naziv_jezika;
	}
	public String definicijaPisma() {
		return definicija_pisma;
	}	
	public int duzinaPisma() {
		return duzina_pisma;
	}	
		
	public char karakter(int pozicija) {
		return tekst.charAt(pozicija);
	}
	public String segment(int pocetak, int kraj) {
		return tekst.substring(pocetak, kraj + 1);
	}
	public int duzina() {
		return tekst.length();
	}
	public int pojavljivanja(char karakter) {
		return broj_pojavljivanja[indeksKaraktera(karakter)];
	}
	public int pojavljivanja(int indeks) {
		return broj_pojavljivanja[indeks];
	}
	public double verovatnoce(char karakter) {
		return (double)pojavljivanja(indeksKaraktera(karakter)) / duzina();		
	}
	public double verovatnoce(int indeks) {
		return (double)pojavljivanja(indeks) / duzina();		
	}	

	public double indeksKoincidencije() {
		double indeks = 0;
		if (duzina() > 0) {
			for (int i = 0; i < duzinaPisma(); i++) {		
				indeks = indeks + (((double)pojavljivanja(i) / (double)duzina()) * ((double)(pojavljivanja(i) - 1) / (double)(duzina() - 1)));			
			}			
		}
		return indeks;
	}
	public double indeksKoincidencije(Dokument dokument) {
		double indeks = 0;
		for (int i = 0; i < duzinaPisma(); i++) {		
			indeks = indeks + (((double)pojavljivanja(i) / (double)duzina()) * ((double)dokument.pojavljivanja(i) / (double)dokument.duzina()));		
		}	
		return indeks;
	}
	public boolean proveriIndeksKoincidencije() {
		double indeks_koincidencije;
		
		indeks_koincidencije = indeksKoincidencije();
		if ((indeks_koincidencije >= interval_poverenja_donja_granica) & (indeks_koincidencije <= interval_poverenja_gornja_granica)) {
			return true;
		} else {
			return false;
		}
		
	}
	
	public Dokument obicnoSifrovanje(char kljuc, boolean sifrovanje) {		
		Dokument d = new Dokument(nazivJezika(), definicijaPisma(), Double.toString(interval_poverenja_donja_granica), Double.toString(interval_poverenja_gornja_granica));
		for (int i = 0; i < duzina(); i++) {
			if (sifrovanje) {
				d.dodajTekst(karakterIndeksa((indeksKaraktera(tekst.charAt(i)) + indeksKaraktera(kljuc)) % duzinaPisma()));	
			} else {
				d.dodajTekst(karakterIndeksa((indeksKaraktera(tekst.charAt(i)) - indeksKaraktera(kljuc) + duzinaPisma()) % duzinaPisma()));
			}
		}		
		return d;
	}
	public Dokument viznerovoSifrovanje(String kljuc, boolean sifrovanje, JProgressBar jProgressBar) {
		if (kljuc.length() == 0) {
			return this;
			
		} else {		
			Dokument d = new Dokument(nazivJezika(), definicijaPisma(), Double.toString(interval_poverenja_donja_granica), Double.toString(interval_poverenja_gornja_granica));
			StringBuffer sb = new StringBuffer();
			while (sb.length() < duzina()) sb.append(kljuc);
			if (jProgressBar != null) {
				jProgressBar.setMaximum(duzina() - 1);
				jProgressBar.setValue(0);			
			}
			for (int i = 0; i < duzina(); i++) {
				if (sifrovanje) {
					d.dodajTekst(karakterIndeksa((indeksKaraktera(tekst.charAt(i)) + indeksKaraktera(sb.charAt(i))) % duzinaPisma()));	
				} else {
					d.dodajTekst(karakterIndeksa((indeksKaraktera(tekst.charAt(i)) - indeksKaraktera(sb.charAt(i)) + duzinaPisma()) % duzinaPisma()));
				}
				if (jProgressBar != null) {				
					updateStatus(jProgressBar, i);
				}
			} 
			return d;
		}
	}

	public Dokument kopirajDokument() {		
		Dokument d = new Dokument(nazivJezika(), definicijaPisma(), Double.toString(interval_poverenja_donja_granica), Double.toString(interval_poverenja_gornja_granica)); 
		d.dodajTekst(tekst);
		return d;
	}
	
	public boolean uporediDokument(Dokument dokument) {	
		if (dokument == null) {
			return false;
		}
		
		if (duzina() != dokument.duzina()) {
			return false;
		}
	
		for (int i = 0; i < duzina(); i++) {
			if (karakter(i) != dokument.karakter(i)) {
				return false;
			}
		}
		
		return true;
	}
	
	public Dokument[] izdeliDokument(int broj_kolona) {
		Dokument d[] = new Dokument[broj_kolona];
		for (int i = 0; i < broj_kolona; i++) {
			d[i] = new Dokument(nazivJezika(), definicijaPisma(), Double.toString(interval_poverenja_donja_granica), Double.toString(interval_poverenja_gornja_granica));
		}
		
		for (int i = 0; i < broj_kolona; i++) {
			for (int j = i; j < duzina(); j += broj_kolona) {
				d[i].dodajTekst(karakter(j));
			}
		}
		
		return d;
	}
	
	public Dokument skratiDokument(int duzina) {
		
		Dokument d;
		
		d = new Dokument(nazivJezika(), definicijaPisma(), Double.toString(interval_poverenja_donja_granica), Double.toString(interval_poverenja_gornja_granica));
		d.dodajTekst(segment(0, duzina - 1));
			
		return d;
	}	

	public double[][] odrediIndekseKoincidencije(int minimum, int maksimum) {
		double indeksi_koincidencije[][];
						
		if ((minimum > 0) &(minimum < maksimum)) {
			indeksi_koincidencije = new double[maksimum - minimum + 1][];			
			for (int i = 0; i < maksimum - minimum + 1; i++) {
				Dokument d[] = izdeliDokument(minimum + i);
				indeksi_koincidencije[i] = new double[d.length];
				
				for (int j = 0; j < d.length; j++) {
					indeksi_koincidencije[i][j] = d[j].indeksKoincidencije();
				}
			}			
		}
		else
		{
			indeksi_koincidencije = new double[0][0];
		}
		
		return indeksi_koincidencije;
	}

	public double[] odrediProsekeIndeksaKoincidencije(int minimum, int maksimum) {
		double indeksi_koincidencije[][] = odrediIndekseKoincidencije(minimum, maksimum);
		double proseci_indeksa_koincidencije[] = new double[indeksi_koincidencije.length];
		for (int i = 0; i < proseci_indeksa_koincidencije.length; i++) proseci_indeksa_koincidencije[i] = 0;
		
		for (int i = 0; i < indeksi_koincidencije.length; i++) {
			if (indeksi_koincidencije[i] != null) {
				double suma_indeksa_koincidencije = 0;
				for (int j = 0; j < indeksi_koincidencije[i].length; j++) {
					suma_indeksa_koincidencije += indeksi_koincidencije[i][j];
				}
				if (indeksi_koincidencije[i].length > 0) { 
					proseci_indeksa_koincidencije[i] = suma_indeksa_koincidencije / indeksi_koincidencije[i].length;
				}				
			}
		}
		
		return proseci_indeksa_koincidencije;
	}
	
	public double[] odrediOdstupanjaIndeksaKoincidencije(int minimum, int maksimum, Dokument tipicanDokument) {
		double indeksi_koincidencije[][] = odrediIndekseKoincidencije(minimum, maksimum);		
		double tipican_indeks_koincidencije = tipicanDokument.indeksKoincidencije();		

		double odstupanja_indeksa_koincidencije[] = new double[indeksi_koincidencije.length];		
		for (int i = 0; i < indeksi_koincidencije.length; i++) {
			if (indeksi_koincidencije[i] != null) {
				double odstupanje_indeksa_koincidencije = 0;
				for (int j = 0; j < indeksi_koincidencije[i].length; j++) {
					odstupanje_indeksa_koincidencije += Math.abs(tipican_indeks_koincidencije - indeksi_koincidencije[i][j]);
				}
				if (indeksi_koincidencije[i].length > 0) { 
					odstupanja_indeksa_koincidencije[i] = odstupanje_indeksa_koincidencije / indeksi_koincidencije[i].length;
				}				
			}
		}
		
		return odstupanja_indeksa_koincidencije;
	}
	
	public double[] odrediKvadrateOdstupanjaIndeksaKoincidencije(int minimum, int maksimum, Dokument tipicanDokument){
		double indeksi_koincidencije[][] = odrediIndekseKoincidencije(minimum, maksimum);
		double tipican_indeks_koincidencije=tipicanDokument.indeksKoincidencije();
		
		double kvadrati_odstupanja_indeksa_koincidencije[] = new double[indeksi_koincidencije.length];
		
		for (int i = 0; i < indeksi_koincidencije.length; i++) {
			if (indeksi_koincidencije[i] != null) {
				double odstupanje_indeksa_koincidencije = 0;
				for (int j = 0; j < indeksi_koincidencije[i].length; j++) {
					odstupanje_indeksa_koincidencije += (Math.abs(tipican_indeks_koincidencije - indeksi_koincidencije[i][j])*Math.abs(tipican_indeks_koincidencije - indeksi_koincidencije[i][j]));
				}
				if (indeksi_koincidencije[i].length > 0) { 
					kvadrati_odstupanja_indeksa_koincidencije[i] = odstupanje_indeksa_koincidencije / indeksi_koincidencije[i].length;
				}				
			}
		}
		return kvadrati_odstupanja_indeksa_koincidencije;
	}
	
	public int pretpostaviDuzinuViznerovogKljuca(int minimum, int maksimum, Dokument tipican_dokument, int metoda) {

		int verovatna_duzina_kljuca = 0;
		
		double odstupanja_indeksa_koincidencije[] = null;
		double proseci_indeksa_koincidencije[] = null;
		
		switch (metoda) {
		case 1:
			odstupanja_indeksa_koincidencije = odrediOdstupanjaIndeksaKoincidencije(minimum, maksimum, tipican_dokument);			
			break;
		case 2:
			odstupanja_indeksa_koincidencije = odrediKvadrateOdstupanjaIndeksaKoincidencije(minimum, maksimum, tipican_dokument);			
			break;
		case 3:
			proseci_indeksa_koincidencije = odrediProsekeIndeksaKoincidencije(minimum, maksimum);			
			break;			 			
		default:
			odstupanja_indeksa_koincidencije = odrediOdstupanjaIndeksaKoincidencije(minimum, maksimum, tipican_dokument);
			break;			
		}
		
		if (odstupanja_indeksa_koincidencije != null) {
			double minimalno_odstupanje_indeksa_koincidencije = Double.MAX_VALUE;
			for (int i = 0; i < odstupanja_indeksa_koincidencije.length; i++) {
				if (odstupanja_indeksa_koincidencije[i] < minimalno_odstupanje_indeksa_koincidencije) {
					minimalno_odstupanje_indeksa_koincidencije = odstupanja_indeksa_koincidencije[i];
					verovatna_duzina_kljuca = minimum + i;
				}
			}
		} else if (proseci_indeksa_koincidencije != null) {			
			double maksimalni_prosek_indeksa_koincidencije = 0;
			for (int i = 0; i < proseci_indeksa_koincidencije.length; i++) {
				if (proseci_indeksa_koincidencije[i] > maksimalni_prosek_indeksa_koincidencije) {
					maksimalni_prosek_indeksa_koincidencije = proseci_indeksa_koincidencije[i];
					verovatna_duzina_kljuca = i;
				}
			}
			verovatna_duzina_kljuca = verovatna_duzina_kljuca + 1;
		}
			
		return verovatna_duzina_kljuca;
	}	
	
	public Object[][] pretpostaviObicanKljuc(Dokument tipican_dokument) {
		Object[][] obican_kljuc = null;
		
		if (duzina() != 0) {
			obican_kljuc = new Object[2][];	
			obican_kljuc[0] = new Character[duzinaPisma()];
			obican_kljuc[1] = new BigDecimal[duzinaPisma()];
			
			for (int i = 0; i < duzinaPisma(); i++) {	
				Dokument d = obicnoSifrovanje(karakterIndeksa(i), false);

				obican_kljuc[0][i] = new Character(karakterIndeksa(i));
				obican_kljuc[1][i] = new BigDecimal(d.indeksKoincidencije(tipican_dokument));			
			}
			
			Character c;
			BigDecimal d;		
			for (int i = 0; i < duzinaPisma() - 1; i++) {
				for (int j = i + 1; j < duzinaPisma(); j++) {				
					if (((BigDecimal)obican_kljuc[1][i]).doubleValue() < ((BigDecimal)obican_kljuc[1][j]).doubleValue()) {
						c = (Character)obican_kljuc[0][i];
						obican_kljuc[0][i] = obican_kljuc[0][j];
						obican_kljuc[0][j] = c;
						
						d = (BigDecimal)obican_kljuc[1][i];
						obican_kljuc[1][i] = obican_kljuc[1][j];
						obican_kljuc[1][j] = d;
					} 
				}
			}			
		}
		
		return obican_kljuc;			
	}
	public char najverovatnijiObicanKljuc(Dokument tipican_dokument) {
		Object[][] obican_kljuc;
		
		obican_kljuc = pretpostaviObicanKljuc(tipican_dokument);
		return ((Character)obican_kljuc[0][0]).charValue();
	}
	public Object[][][] pretpostaviVizenrovKljuc(int duzina_kljuca, Dokument tipican_dokument) {
		Object[][][] viznerov_kljuc = null;
		Object[][] obican_kljuc;
		
		if (duzina() != 0) {		
			viznerov_kljuc = new Object[2][][];	
			viznerov_kljuc[0] = new Character[duzina_kljuca][duzinaPisma()];
			viznerov_kljuc[1] = new BigDecimal[duzina_kljuca][duzinaPisma()];		
		
			Dokument d[] = izdeliDokument(duzina_kljuca);		
			for  (int i = 0; i < duzina_kljuca; i++) {
				obican_kljuc = d[i].pretpostaviObicanKljuc(tipican_dokument);
				
				viznerov_kljuc[0][i] = obican_kljuc[0];
				viznerov_kljuc[1][i] = obican_kljuc[1];
			}			
		}
			
		return viznerov_kljuc;
	}	
	public String najverovatnijiVizenrovKljuc(int duzina_kljuca, Dokument tipican_dokument) {
		Object[][][] viznerov_kljuc;
		String najverovatniji_viznerov_kljuc;
		
		viznerov_kljuc = pretpostaviVizenrovKljuc(duzina_kljuca, tipican_dokument);
		najverovatniji_viznerov_kljuc = "";
		for (int i = 0; i < viznerov_kljuc[0].length; i++) {
			najverovatniji_viznerov_kljuc += viznerov_kljuc[0][i][0];
		}		
		
		return najverovatniji_viznerov_kljuc;
	}
			
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(tekst);			
		for (int i = 0; i < sb.length(); i++) {
			if ((i % 80 == 0) && (i > 0)) sb.insert(i, '\n');
		}		
		return sb.toString();		
	}
	
	public String toInfo() {	
		String i;
		if (proveriIndeksKoincidencije() == true) {
			i = "nema znaajnih odstupanja";
		} else {
			i = "ima znaajnih odstupanja";			
		}
		
		String s = "";
		s += "PRIMENJENI JEZIK [" + nazivJezika() + "]\n" + 
			 "DUZINA PISMA [" + duzinaPisma() + "]\n" +			
			 "DUZINA TEKSTA [" + duzina() + "]\n" +
			 "INDEKS KOINCIDENCIJE [" + Dokument.doubleToString(indeksKoincidencije(), IK) + "]\n" +
			 "PRIRODNI JEZIK [" + i + "]"; 
		
		return s;		
	}
	
	public String toStatistics() {
		double d;
		String s = "";
		for (int i = 0; i < duzinaPisma(); i++) {
			s += karakterIndeksa(i);
			if (duzina() > 0) {
				d = verovatnoce(i);				
				s += " = " + doubleToString(d, P) + "\n";
			} else {
				s += " = 0\n";
			}
		}
		return s;
	}
	
	public void toFile(File tekst, String charsetName) {
		try {
			FileOutputStream fos = new FileOutputStream(tekst);
			OutputStreamWriter osw = new OutputStreamWriter(fos, charsetName);		
			BufferedWriter bw = new BufferedWriter(osw);
			
			bw.write(toString());
			
			bw.close();
			osw.close();
			fos.close();
		} catch (IOException e) {
			System.out.println(MSG_IO_EXCEPTION_WRITE);				
		}		
	}
	
	private void ucitajJezik(String naziv_jezika, String definicija_pisma, String interval_poverenja_donja_granica, String interval_poverenja_gornja_granica) {
		this.naziv_jezika = naziv_jezika;
		this.definicija_pisma = definicija_pisma;
		this.duzina_pisma = definicija_pisma.length() / 2;
		this.mala_slova = new char[duzina_pisma];
		this.velika_slova = new char[duzina_pisma];
		this.broj_pojavljivanja = new int[duzina_pisma];
		
		if (interval_poverenja_donja_granica != null) {
			this.interval_poverenja_donja_granica = (new Double(interval_poverenja_donja_granica)).doubleValue();
		} else {
			this.interval_poverenja_donja_granica = 0;
		}
		if (interval_poverenja_gornja_granica != null) {
			this.interval_poverenja_gornja_granica = (new Double(interval_poverenja_gornja_granica)).doubleValue();
		} else {
			this.interval_poverenja_gornja_granica = 1;
		}
		
		for (int i = 0; i < duzina_pisma; i++) {				
			mala_slova[i] = definicija_pisma.charAt(i*2);
			velika_slova[i] = definicija_pisma.charAt(i*2 + 1);
			broj_pojavljivanja[i] = 0;
		}
		
		tekst = new StringBuffer();		
	}
	private String obradiKarakter(char karakter) {
		String s = "";
		for (int i = 0; i < duzinaPisma(); i++) {
			if (karakter == velika_slova[i]) karakter = mala_slova[i];
			if (karakter == mala_slova[i]) s += karakter;
		}
		return s;
	}
	private int indeksKaraktera(char karakter) {
		for (int i = 0; i < duzinaPisma(); i++) {
			if (karakter == mala_slova[i]) return i;
		}
		return -1;
	}
	private char karakterIndeksa(int indeks) {
		if ((indeks >= 0) && (indeks < duzinaPisma())) {
			return mala_slova[indeks];
		} else {
			return '#';
		}
	}	
	private void popuniPojavljivanja() {
		for (int i = 0; i < duzinaPisma(); i++) {
			broj_pojavljivanja[i] = 0;
		}
		for (int i = 0; i < duzina(); i++) {
			broj_pojavljivanja[indeksKaraktera(tekst.charAt(i))]++;
		}
	}	

    private void updateStatus(final JProgressBar jProgressBar, final int i) {
        Runnable doSetProgressBarValue = new Runnable() {
            public void run() {
                jProgressBar.setValue(i);
            }
        };
        SwingUtilities.invokeLater(doSetProgressBarValue);
    }	

    public static String doubleToString(double vrednost, int decimale) {
    	String s;
    	
    	s = "";
    	vrednost = (new BigDecimal(vrednost).setScale(decimale, RoundingMode.HALF_UP)).doubleValue();    	
    	s = vrednost + "";
    	
    	while ((s.length() - s.indexOf(".") - 1) < decimale) {
    		s += "0";
    	}
    	
    	return s;
    }
    
    public static int generisiSlucajanBroj(int broj_od, int broj_do) {
		Random random;
		int a,b,c,d;
		int broj;
		
		random = new Random();
		a = random.nextInt();
		b = random.nextInt();
		c = random.nextInt();
		d = random.nextInt();
		
		broj = Math.abs(a ^ b ^ c ^ d);
		broj = broj_od + (broj % Math.abs(broj_do - broj_od));
		return broj;		
	}    
    
	public String generisiSlucajanKljuc(int duzinaKljuca) {
		int slucajan_indeks;
		String slucajan_kljuc;
		
		slucajan_kljuc = "";
		for (int i = 0; i < duzinaKljuca; i++) {
			slucajan_indeks = (int)generisiSlucajanBroj(0, duzinaPisma());
			slucajan_kljuc += karakterIndeksa(slucajan_indeks);
		}

		return slucajan_kljuc;
	}
	
    public static double odrediSrednjuVrednostUzorka(double[] uzorak) {
    	double srednja_vrednost_uzorka;
    	
    	srednja_vrednost_uzorka = 0;
    	for (int i = 0; i < uzorak.length; i++) {
    		srednja_vrednost_uzorka += uzorak[i];
    	}
    	srednja_vrednost_uzorka = srednja_vrednost_uzorka / (double)uzorak.length;
    	
    	return srednja_vrednost_uzorka;
    }
    
    public static double odrediVarijansuUzorka(double[] uzorak) {
    	double srednja_vrednost_uzroka;
    	double varijansa_uzorka;
    	
    	srednja_vrednost_uzroka = odrediSrednjuVrednostUzorka(uzorak);
    	
    	varijansa_uzorka = 0;
    	for (int i = 0; i < uzorak.length; i++) {
    		varijansa_uzorka += (uzorak[i] * uzorak[i]);    		
    	}
    	varijansa_uzorka = varijansa_uzorka / (double)(uzorak.length - 1);
    	varijansa_uzorka = varijansa_uzorka - (srednja_vrednost_uzroka * srednja_vrednost_uzroka * uzorak.length / (double)(uzorak.length - 1));
    	
    	return varijansa_uzorka;
    }
    
    public static double odrediStandardnuDevijacijuUzorka(double[] uzorak) {
    	double varijansa_uzorka;
    	double standardna_devijacija_uzorka;
    	
    	varijansa_uzorka = odrediVarijansuUzorka(uzorak);    	
    	standardna_devijacija_uzorka = Math.sqrt(varijansa_uzorka);
    	
    	return standardna_devijacija_uzorka;
    }
    
    public static double[] odrediIntervalPoverenja(double[] uzorak) {
    	double srednja_vrednost_uzroka;
    	double standardna_devijacija_uzorka;    	    	
    	double kvantil;    	
    	double delta;
    	double[] interval_poverenja_99;

    	srednja_vrednost_uzroka = odrediSrednjuVrednostUzorka(uzorak);
    	standardna_devijacija_uzorka = odrediStandardnuDevijacijuUzorka(uzorak);
    	
    	kvantil = 3.106;
    	interval_poverenja_99 = new double[2];
    	delta = kvantil * standardna_devijacija_uzorka / Math.sqrt(uzorak.length);
    	interval_poverenja_99[0] = srednja_vrednost_uzroka - delta;
    	interval_poverenja_99[1] = srednja_vrednost_uzroka + delta;
    	
    	return interval_poverenja_99;
    }
	
	public static int izracunajEksponent(double broj) {
		
		int eksponent = 0;
		
		if (broj < 1) {
			do {
				broj = broj * (double)10;
				eksponent = eksponent - 1;
			} while (broj < 1);
		} 
		/*else if (broj >= 10) {
			do {
				broj = broj / (double)10;
				eksponent = eksponent + 1; 
			} while (broj >= 10);
		}*/
		
		return eksponent;
	}
    	
    public static void statistike_main(int brojRedova, int maksimalnaDuzinaKljuca, int brojEksperimenata, int upotrebljeniJezik, int upotrebljenaMetoda) {

    	File jezik = null;
    	Dokument baza = null;
    	Dokument tipican_dokument = null;
    	
		int duzina_teksta;
		int pocetak_poddokumenta;
		int kraj_poddokumenta;
		
		int duzina_kljuca;
		String kljuc;
		Dokument tekst;
		
		Dokument sifrat;
		
		int _duzina_kljuca;
		String _kljuc;
		Dokument _tekst;
		
		int brojPogodaka;
		double uspesnost;		
		double rezultat[][];    	
    	   	
    	if (upotrebljeniJezik == 1) {
    	    
    		jezik = new File(System.getProperty("user.dir") + "\\#jezici\\english.txt");    	
    	    baza = new Dokument(jezik, "UTF-16");
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\Aesops Fables.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\Clockwork Orange.txt"));		
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\Hitchhikers.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\Robinson Crusoe.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\20000 leagues under the sea.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\Harry Potter5.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\In Search of the Castaways.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\Rama Revealed.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\Digital Fortress.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\From Revolution to Reconstruction Documents Paine Rights of .txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\The Dark Tower Books 5.txt"));
    	    baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\english\\The Hobit.txt"));
    	    
    	} else if (upotrebljeniJezik == 2) {
    		
    		jezik = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica.txt");		
    		baza = new Dokument(jezik, "UTF-16");
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Carobnjak iz Oza.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Da Vincijev Kod.txt"));		
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Dr BloodMoney.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Kad Su Cvetale Tikve.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Magarece Godine.txt"));		
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Nusic.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Protokoli.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Seobe.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Sidarta.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Slovenska Mitologija.txt"));    		
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Za i Protiv Vuka.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Zakoni Prosveta.txt"));    
    		
    	} else if (upotrebljeniJezik == 3) {
    		
    		jezik = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica.txt");		
    		baza = new Dokument(jezik, "UTF-16");
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Anticka Srbija.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Carobnjak iz Oza.txt"));		
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Da Vincijev kod.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Dr Blood Money.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\1984.txt"));		
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Nusic.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Kad su cvetale tikve.txt"));    		
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Seobe.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Sidarta.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Ubistvo u Orijent ekspresu.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Una.txt"));
    		baza.dodajTekst(new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Zakoni prosveta.txt"));    		
    	}
    		
		tipican_dokument = baza.kopirajDokument();
					
		rezultat = new double[brojRedova][maksimalnaDuzinaKljuca];
		for (int i = 1; i <= brojRedova; i++) {
			for (int j = 1; j <= maksimalnaDuzinaKljuca; j++) {
				
				brojPogodaka = 0;
				for (int k = 1; k <= brojEksperimenata; k++) {
								
					duzina_teksta = 1;
					for (int l = 0; l < i - 1; l++) {
						duzina_teksta = duzina_teksta * 2; 
					}					
					duzina_teksta = duzina_teksta * 32;
					
					pocetak_poddokumenta = Dokument.generisiSlucajanBroj(0, baza.duzina() - duzina_teksta - 1);
					kraj_poddokumenta = pocetak_poddokumenta + duzina_teksta - 1;			
					tekst = new Dokument(jezik, "UTF-16");			
					
					duzina_kljuca = j;				
					kljuc = tekst.generisiSlucajanKljuc(duzina_kljuca);
					tekst.dodajTekst(baza.segment(pocetak_poddokumenta, kraj_poddokumenta));
					
					sifrat = tekst.viznerovoSifrovanje(kljuc, true, null);
					
					_duzina_kljuca = sifrat.pretpostaviDuzinuViznerovogKljuca(1, maksimalnaDuzinaKljuca, tipican_dokument, upotrebljenaMetoda);
					_kljuc = sifrat.najverovatnijiVizenrovKljuc(_duzina_kljuca, tipican_dokument);
					_tekst = sifrat.viznerovoSifrovanje(_kljuc, false, null);
													
					if (tekst.uporediDokument(_tekst) == true) {
						brojPogodaka++;
					}
					
					System.out.println("tekst[2^" + i + "*32] kljuc[" + j + "] eksperiment[" + k + "] pogodaka[" + brojPogodaka + "]");					
				}
				uspesnost = (double)brojPogodaka / (double)brojEksperimenata;
				
				rezultat[i - 1][j - 1] = uspesnost;				
			}
		}
		System.out.println();
		
		for (int i = 0; i < brojRedova; i++) {
			for (int j = 0; j < maksimalnaDuzinaKljuca; j++) {
				System.out.print(Dokument.doubleToString(rezultat[i][j], 4) + " ");
			}
			System.out.println();
		}
		
		System.out.println("[JEZIK]");
		if(upotrebljeniJezik == 1)
			System.out.println("english");
		else if (upotrebljeniJezik == 2)
			System.out.println("srpski (latinica)");
		else if (upotrebljeniJezik == 3)
			System.out.println("srpski (cirilica)");
		System.out.println();
		
		System.out.println("[METODA]");
		if(upotrebljenaMetoda == 1)
			System.out.println("prosek odstupanja");
		else if (upotrebljenaMetoda == 2)
			System.out.println("srednje kvadratno odstupanje");
		else if (upotrebljenaMetoda == 3)
			System.out.println("maksimalni indeksi koincidencije");
		System.out.println();
		
		System.out.println("[PODACI O BAZI]");
		System.out.println(baza.toInfo());
		System.out.println();
		
		System.out.println("[TIPICAN DOKUMENT]");
		System.out.println(tipican_dokument.toInfo());    	    
    }
    
    public static void zaokruzivanja_main(int upotrebljeniJezik) {
    	 	
    	File jezik;    	
    	File[] putanje;
    	Dokument[] dokumenti;
    	
    	int duzina_pisma;
    	int duzina_uzorka;
    	
    	//promenljive za ocenu varijanse indeksa koinciencije
    	double[] indeksi;
    	double indeksi_m;
    	double indeksi_s2;
    	double indeksi_s;
    	double[] indeksi_ip;
    	
    	//promenljive za ocenu varijanse slova primenjenog jezika   	    	
    	double[][] slova;
    	double[] slova_m;
    	double[] slova_s2;
    	double[] slova_s;
    	double[][] slova_ip;
    	
    	jezik = null;
    	putanje = new File[12];
    	
    	if (upotrebljeniJezik == 1) {
        	jezik = new File(System.getProperty("user.dir") + "\\#jezici\\english.txt");
        	
        	putanje[0] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\20000 leagues under the sea.txt");
        	putanje[1] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\Aesops Fables.txt");
        	putanje[2] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\Clockwork Orange.txt");
        	putanje[3] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\Digital Fortress.txt");		
        	putanje[4] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\Harry Potter5.txt");
        	putanje[5] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\Hitchhikers.txt");
        	putanje[6] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\In Search of the Castaways.txt");
        	putanje[7] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\Rama Revealed.txt");
        	putanje[8] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\Robinson Crusoe.txt");    		
        	putanje[9] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\The Hobit.txt");
        	putanje[10] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\From Revolution to Reconstruction Documents Paine Rights of .txt");    	
        	putanje[11] = new File(System.getProperty("user.dir") + "\\#jezici\\english\\The Dark Tower Books 5.txt");
    	
    	} else if (upotrebljeniJezik == 2) {
    		
    		jezik = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica.txt");
    	
    		putanje[0] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Carobnjak iz Oza.txt");
    		putanje[1] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Da Vincijev Kod.txt");
    		putanje[2] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Kad Su Cvetale Tikve.txt");
    		putanje[3] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Magarece Godine.txt");		
    		putanje[4] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Nusic.txt");
    		putanje[5] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Protokoli.txt");
    		putanje[6] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Seobe.txt");
    		putanje[7] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Sidarta.txt");
    		putanje[8] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Slovenska Mitologija.txt");    		
    		putanje[9] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Za i Protiv Vuka.txt");
    		putanje[10] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Una.txt");    	
    		putanje[11] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_latinica\\Zakoni Prosveta.txt");    	
    	
    	} else if (upotrebljeniJezik == 3) {
    	
    		jezik = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica.txt");
    	
    		putanje[0] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Dr Blood Money.txt");
    		putanje[1] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Da Vincijev Kod.txt");
    		putanje[2] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\1984.txt");
    		putanje[3] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Zakoni Prosveta.txt");		
    		putanje[4] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Nusic.txt");
    		putanje[5] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Kad su cvetale tikve.txt");
    		putanje[6] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Seobe.txt");
    		putanje[7] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Ubistvo u Orijent ekspresu.txt");
    		putanje[8] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Sidarta.txt");    		
    		putanje[9] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Anticka Srbija.txt");
    		putanje[10] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Una.txt");    	
    		putanje[11] = new File(System.getProperty("user.dir") + "\\#jezici\\srpski_cirilica\\Carobnjak iz Oza.txt");    	
    	}
        	
    	duzina_uzorka = putanje.length;
    	
    	System.out.println("[UCITAVANJE]");    	    	
    	//kreiranje dokumenata iste duzine na istom jeziku
    	dokumenti = new Dokument[12];
    	for (int i = 0; i < 12; i++) {    		
    		dokumenti[i] = new Dokument(jezik, "UTF-16");
    		dokumenti[i].dodajTekst(putanje[i]);
    		dokumenti[i] = dokumenti[i].skratiDokument(50000);
    		
    		System.out.println("dokument[" + i + "] " + dokumenti[i].duzina());    		
       	}   	
    	duzina_pisma = dokumenti[0].duzinaPisma();

    	System.out.println();    	
    	System.out.println("[UZIMANJE UZORKA]");
    	//racunanje ideksa koincidencije za svaki dokument posebno  	
    	indeksi = new double[duzina_uzorka];
    	for (int i = 0; i < duzina_uzorka; i++) {
			indeksi[i] = dokumenti[i].indeksKoincidencije();
			System.out.println("indeksi[" + i + "] = " + indeksi[i]);    		
    	}
		System.out.println();    	
    	//racunanje verovatnoca pojavljivanja svakog slova
		slova = new double[duzina_pisma][duzina_uzorka];
    	for (int i = 0; i < duzina_pisma; i++)
		{			
			for (int j = 0; j < duzina_uzorka; j++) {
				slova[i][j] = dokumenti[j].verovatnoce(i);
				System.out.println("slova[" + i + "][" + j + "] = " + slova[i][j]);
			}
			System.out.println();			
		}
    	
    	System.out.println();    		
    	System.out.println("[MI SA KAPICOM]");    	
    	//ocena matematickog ocekivanja za indekse koincidencija	
    	indeksi_m = odrediSrednjuVrednostUzorka(indeksi);
		System.out.println("indeksi_m = " + indeksi_m);    	
		//ocena matematickog ocekivanja za slova azbuke	
		slova_m = new double[duzina_pisma];
		for (int i = 0; i < duzina_pisma; i++) {
			slova_m[i] = odrediSrednjuVrednostUzorka(slova[i]);
			System.out.println(slova_m[i]);			
		}
		
		System.out.println();				
    	System.out.println("[S NA KVADRAT]");
		   	
    	indeksi_s2 = odrediVarijansuUzorka(indeksi);
		System.out.println("indeksi_s2 = " + indeksi_s2);    	
		
		slova_s2 = new double[duzina_pisma];
		for (int i = 0; i < duzina_pisma; i++) {
			slova_s2[i] = odrediVarijansuUzorka(slova[i]);			
			System.out.println("slova_s2[" + i + "] = " + slova_s2[i]);
		}
					
		System.out.println();				
    	System.out.println("[S]");		
    	
    	indeksi_s = odrediStandardnuDevijacijuUzorka(indeksi);
		System.out.println("indeksi_s = " + indeksi_s + " (eksponent = " + Dokument.izracunajEksponent(indeksi_s) + ")");    	
    	
		slova_s = new double[duzina_pisma];
		for (int i = 0; i < duzina_pisma; i++) {
			slova_s[i] = Math.sqrt(slova_s2[i]);
			System.out.println("slova_s[" + i + "] = " + slova_s[i] + " (eksponent = " + Dokument.izracunajEksponent(slova_s[i]) + ")");			
		}
		
		System.out.println();				
    	System.out.println("[INTERVAL POVERENJA]");
    	
    	indeksi_ip = odrediIntervalPoverenja(indeksi);
    	System.out.println("indeksi_ip[donja_granica] = " + indeksi_ip[0] + ", indeksi_ip[gornja_granica] = " + indeksi_ip[1]);
    	
    	slova_ip = new double[duzina_pisma][2];
    	for (int i = 0; i < duzina_pisma; i++) {
    		slova_ip[i] = odrediIntervalPoverenja(slova[i]);
    		System.out.println("slova_ip[" + i + "][donja_granica] = " + slova_ip[i][0] + ", slova_ip[" + i + "][gornja_granica] = " + slova_ip[i][1]);
    	}
    }
    
	public static void main(String[] args) {		
				
		Dokument.statistike_main(7, 16, 100, 1, 2);
		//Dokument.zaokruzivanja_main(1);
	}	
}